"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAceCompatAutocompleter = exports.createCompletionResultForIdPrefix = exports.resolveTokenAtCursor = exports.ID_REGEX = exports.completeWordsInString = exports.mapMongoDBCompletionToCodemirrorCompletion = void 0;
const autocomplete_1 = require("@codemirror/autocomplete");
const autocomplete_2 = require("@codemirror/autocomplete");
const language_1 = require("@codemirror/language");
const autocompleter_1 = require("../autocompleter");
function mapMongoDBCompletionToCodemirrorCompletion(completion, escape = 'invalid') {
    var _a;
    const cmCompletion = {
        label: completion.value,
        apply: escape === 'never'
            ? completion.value
            : (0, autocompleter_1.wrapField)(completion.value, escape === 'always'),
        detail: ((_a = completion.meta) === null || _a === void 0 ? void 0 : _a.startsWith('field')) ? 'field' : completion.meta,
        info: completion.description,
    };
    if (completion.snippet) {
        return (0, autocomplete_2.snippetCompletion)(completion.snippet, cmCompletion);
    }
    return cmCompletion;
}
exports.mapMongoDBCompletionToCodemirrorCompletion = mapMongoDBCompletionToCodemirrorCompletion;
exports.completeWordsInString = (0, autocomplete_1.ifIn)(['String'], autocomplete_1.completeAnyWord);
exports.ID_REGEX = /[a-zA-Z_0-9$\-\u00A2-\u2000\u2070-\uFFFF]+/;
function resolveTokenAtCursor(context) {
    return (0, language_1.syntaxTree)(context.state).resolveInner(context.pos, -1);
}
exports.resolveTokenAtCursor = resolveTokenAtCursor;
function createCompletionResultForIdPrefix({ prefix, completions, escape, }) {
    return {
        from: prefix.from,
        to: prefix.to,
        options: completions.map((completion) => {
            return mapMongoDBCompletionToCodemirrorCompletion(completion, escape);
        }),
        validFor: exports.ID_REGEX,
    };
}
exports.createCompletionResultForIdPrefix = createCompletionResultForIdPrefix;
function createAceCompatAutocompleter(completers) {
    return function aceCompatAutocompleter(context) {
        var _a, _b, _c, _d;
        const prefix = context.matchBefore(exports.ID_REGEX);
        const token = resolveTokenAtCursor(context);
        if (!prefix || prefix.text === '') {
            return null;
        }
        if (['BlockComment', 'LineComment'].includes(token.type.name)) {
            return null;
        }
        const opts = { context, prefix, token };
        if (token.type.name === 'String') {
            return (_b = (_a = completers.String) === null || _a === void 0 ? void 0 : _a.call(completers, opts)) !== null && _b !== void 0 ? _b : null;
        }
        return (_d = (_c = completers.IdentifierLike) === null || _c === void 0 ? void 0 : _c.call(completers, opts)) !== null && _d !== void 0 ? _d : null;
    };
}
exports.createAceCompatAutocompleter = createAceCompatAutocompleter;
//# sourceMappingURL=ace-compat-autocompleter.js.map