import React from 'react';
import type { Command } from '@codemirror/view';
import { EditorView } from '@codemirror/view';
import { unfoldAll } from '@codemirror/language';
import type { Diagnostic } from '@codemirror/lint';
import type { CompletionSource } from '@codemirror/autocomplete';
import { Facet } from '@codemirror/state';
import { LanguageSupport } from '@codemirror/language';
declare type EditorLanguage = 'json' | 'javascript';
declare type Annotation = Pick<Diagnostic, 'from' | 'to' | 'severity' | 'message'>;
declare type EditorProps = {
    language?: EditorLanguage;
    onChangeText?: (text: string, event?: any) => void;
    onLoad?: (editor: EditorView) => void;
    darkMode?: boolean;
    showLineNumbers?: boolean;
    showFoldGutter?: boolean;
    highlightActiveLine?: boolean;
    readOnly?: boolean;
    className?: string;
    id?: string;
    'data-testid'?: string;
    annotations?: Annotation[];
    completer?: CompletionSource;
    minLines?: number;
    maxLines?: number;
    lineHeight?: number;
} & ({
    text: string;
    initialText?: never;
} | {
    text?: never;
    initialText: string;
});
export declare const languages: Record<EditorLanguage, () => LanguageSupport>;
export declare const languageName: Facet<EditorLanguage, readonly EditorLanguage[]>;
declare const BaseEditor: React.FunctionComponent<EditorProps> & {
    foldAll: typeof foldAll;
    unfoldAll: typeof unfoldAll;
    copyAll: typeof copyAll;
    prettify: typeof prettify;
};
declare const foldAll: Command;
declare const copyAll: Command;
declare const prettify: Command;
export type { EditorView };
export { BaseEditor as JSONEditor };
//# sourceMappingURL=json-editor.d.ts.map