"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JSONEditor = exports.languageName = exports.languages = void 0;
const react_1 = __importStar(require("react"));
const view_1 = require("@codemirror/view");
const language_1 = require("@codemirror/language");
const commands_1 = require("@codemirror/commands");
const lint_1 = require("@codemirror/lint");
const autocomplete_1 = require("@codemirror/autocomplete");
const compass_components_1 = require("@mongodb-js/compass-components");
const lang_javascript_1 = require("@codemirror/lang-javascript");
const lang_json_1 = require("@codemirror/lang-json");
const state_1 = require("@codemirror/state");
const language_2 = require("@codemirror/language");
const highlight_1 = require("@lezer/highlight");
const polished_1 = require("polished");
const prettify_1 = require("./prettify");
const editorStyle = (0, compass_components_1.css)({
    fontSize: 13,
    fontFamily: compass_components_1.fontFamilies.code,
});
const editorPalette = {
    light: {
        color: compass_components_1.codePalette.light[3],
        backgroundColor: compass_components_1.codePalette.light[0],
        gutterColor: compass_components_1.palette.gray.base,
        gutterBackgroundColor: compass_components_1.palette.gray.light3,
        gutterActiveLineBackgroundColor: (0, polished_1.rgba)(compass_components_1.palette.gray.light2, 0.5),
        gutterFoldButtonColor: compass_components_1.palette.black,
        cursorColor: compass_components_1.palette.gray.base,
        activeLineBackgroundColor: (0, polished_1.rgba)(compass_components_1.palette.gray.light2, 0.5),
        selectionBackgroundColor: compass_components_1.palette.blue.light2,
        bracketBorderColor: compass_components_1.palette.gray.light1,
        infoGutterIconColor: encodeURIComponent(compass_components_1.palette.blue.base),
        warningGutterIconColor: encodeURIComponent(compass_components_1.palette.yellow.base),
        errorGutterIconColor: encodeURIComponent(compass_components_1.palette.red.base),
        foldPlaceholderColor: compass_components_1.palette.gray.base,
        foldPlaceholderBackgroundColor: compass_components_1.palette.gray.light3,
    },
    dark: {
        color: compass_components_1.codePalette.dark[3],
        backgroundColor: compass_components_1.codePalette.dark[0],
        gutterColor: compass_components_1.palette.gray.light3,
        gutterBackgroundColor: compass_components_1.palette.gray.dark4,
        gutterActiveLineBackgroundColor: (0, polished_1.rgba)(compass_components_1.palette.gray.dark2, 0.5),
        gutterFoldButtonColor: compass_components_1.palette.white,
        cursorColor: compass_components_1.palette.green.base,
        activeLineBackgroundColor: (0, polished_1.rgba)(compass_components_1.palette.gray.dark2, 0.5),
        selectionBackgroundColor: compass_components_1.palette.gray.dark1,
        bracketBorderColor: compass_components_1.palette.gray.light1,
        infoGutterIconColor: encodeURIComponent(compass_components_1.palette.blue.light1),
        warningGutterIconColor: encodeURIComponent(compass_components_1.palette.yellow.light2),
        errorGutterIconColor: encodeURIComponent(compass_components_1.palette.red.light1),
        foldPlaceholderColor: compass_components_1.palette.gray.base,
        foldPlaceholderBackgroundColor: compass_components_1.palette.gray.dark3,
    },
};
function getStylesForTheme(theme) {
    return view_1.EditorView.theme({
        '&': {
            color: editorPalette[theme].color,
            backgroundColor: editorPalette[theme].backgroundColor,
        },
        '& .cm-scroller': {
            fontSize: '13px',
            fontFamily: compass_components_1.fontFamilies.code,
        },
        '&.cm-editor.cm-focused': {
            outline: 'none',
        },
        '& .cm-content': {
            caretColor: editorPalette[theme].cursorColor,
        },
        '& .cm-activeLine': {
            background: 'none',
        },
        '&.cm-focused .cm-activeLine': {
            backgroundColor: editorPalette[theme].activeLineBackgroundColor,
        },
        '& .cm-gutters': {
            color: editorPalette[theme].gutterColor,
            backgroundColor: editorPalette[theme].gutterBackgroundColor,
            border: 'none',
        },
        '& .cm-gutters > .cm-gutter:first-child > .cm-gutterElement': {
            paddingLeft: `${compass_components_1.spacing[2]}px`,
        },
        '& .cm-gutters > .cm-gutter:last-child > .cm-gutterElement': {
            paddingRight: `${compass_components_1.spacing[2]}px`,
        },
        '& .cm-gutter-lint': {
            width: 'auto',
        },
        '& .cm-gutter-lint .cm-gutterElement': {
            padding: '0',
        },
        '& .cm-gutter-lint .cm-lint-marker': {
            width: `${compass_components_1.spacing[3]}px`,
            height: `${compass_components_1.spacing[3]}px`,
            padding: '2px',
        },
        '& .cm-gutter-lint .cm-lint-marker-info': {
            content: `url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' fill='none' viewBox='0 0 16 16'%3E%3Cpath fill='${editorPalette[theme].infoGutterIconColor}' fill-rule='evenodd' d='M8 15A7 7 0 1 0 8 1a7 7 0 0 0 0 14ZM9 4a1 1 0 1 1-2 0 1 1 0 0 1 2 0ZM8 6a1 1 0 0 1 1 1v4h.5a.5.5 0 0 1 0 1h-3a.5.5 0 0 1 0-1H7V7h-.5a.5.5 0 0 1 0-1H8Z' clip-rule='evenodd'/%3E%3C/svg%3E%0A")`,
        },
        '& .cm-gutter-lint .cm-lint-marker-warning': {
            content: `url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' fill='none' viewBox='0 0 16 16'%3E%3Cpath fill='${editorPalette[theme].warningGutterIconColor}' fill-rule='evenodd' d='M8.864 2.514a.983.983 0 0 0-1.728 0L1.122 13.539A.987.987 0 0 0 1.986 15h12.028a.987.987 0 0 0 .864-1.461L8.864 2.514ZM7 6a1 1 0 0 1 2 0v4a1 1 0 1 1-2 0V6Zm2 7a1 1 0 1 1-2 0 1 1 0 0 1 2 0Z' clip-rule='evenodd'/%3E%3C/svg%3E%0A")`,
        },
        '& .cm-gutter-lint .cm-lint-marker-error': {
            content: `url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' fill='none' viewBox='0 0 16 16'%3E%3Cpath fill='${editorPalette[theme].errorGutterIconColor}' fill-rule='evenodd' d='M8 15A7 7 0 1 0 8 1a7 7 0 0 0 0 14zm1.414-9.828a1 1 0 1 1 1.414 1.414L9.414 8l1.414 1.414a1 1 0 1 1-1.414 1.414L8 9.414l-1.414 1.414a1 1 0 1 1-1.414-1.414L6.586 8 5.172 6.586a1 1 0 0 1 1.414-1.414L8 6.586l1.414-1.414z' clip-rule='evenodd'/%3E%3C/svg%3E%0A")`,
        },
        '& .cm-activeLineGutter': {
            background: 'none',
        },
        '&.cm-focused .cm-activeLineGutter': {
            backgroundColor: editorPalette[theme].gutterActiveLineBackgroundColor,
        },
        '& .cm-foldPlaceholder': {
            display: 'inline-block',
            border: 'none',
            color: editorPalette[theme].foldPlaceholderColor,
            backgroundColor: editorPalette[theme].foldPlaceholderBackgroundColor,
            boxShadow: `inset 0 0 0 1px ${editorPalette[theme].foldPlaceholderColor}`,
            padding: '0 2px',
        },
        '& .foldMarker': {
            width: `${compass_components_1.spacing[3]}px`,
            height: `${compass_components_1.spacing[3]}px`,
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'center',
            cursor: 'pointer',
        },
        '& .foldMarker > svg': {
            width: '12px',
            height: '12px',
            display: 'block',
            color: editorPalette[theme].gutterFoldButtonColor,
        },
        '& .cm-selectionBackground': {
            backgroundColor: editorPalette[theme].selectionBackgroundColor,
        },
        '&.cm-focused .cm-matchingBracket': {
            background: 'none',
            boxShadow: `0 0 0 1px ${editorPalette[theme].bracketBorderColor}`,
            borderRadius: '2px',
        },
        '& .cm-selectionBackground:first-child': {
            borderTopLeftRadius: '3px',
            borderTopRightRadius: '3px',
        },
        '& .cm-selectionBackground:last-child': {
            borderBottomLeftRadius: '3px',
            borderBottomRightRadius: '3px',
        },
        '& .cm-cursor': {
            borderLeft: '2px solid',
            borderColor: editorPalette[theme].cursorColor,
            marginLeft: '-1px',
        },
    }, { dark: theme === 'dark' });
}
const themeStyles = {
    light: getStylesForTheme('light'),
    dark: getStylesForTheme('dark'),
};
function getHighlightStyleForTheme(theme) {
    return language_2.HighlightStyle.define([
        {
            tag: [
                highlight_1.tags.null,
                highlight_1.tags.controlKeyword,
                highlight_1.tags.operatorKeyword,
                highlight_1.tags.definitionKeyword,
                highlight_1.tags.moduleKeyword,
                highlight_1.tags.keyword,
                highlight_1.tags.modifier,
            ],
            color: compass_components_1.codePalette[theme][10],
        },
        {
            tag: [highlight_1.tags.number, highlight_1.tags.bool, highlight_1.tags.function(highlight_1.tags.variableName), highlight_1.tags.regexp],
            color: compass_components_1.codePalette[theme][9],
        },
        {
            tag: [highlight_1.tags.string, highlight_1.tags.special(highlight_1.tags.string)],
            color: compass_components_1.codePalette[theme][7],
            fontWeight: 600,
        },
        {
            tag: [highlight_1.tags.meta],
            color: compass_components_1.codePalette[theme][6],
        },
        {
            tag: [highlight_1.tags.propertyName, highlight_1.tags.atom, highlight_1.tags.self],
            color: compass_components_1.codePalette[theme][5],
        },
        {
            tag: [
                highlight_1.tags.separator,
                highlight_1.tags.squareBracket,
                highlight_1.tags.brace,
                highlight_1.tags.variableName,
                highlight_1.tags.definition(highlight_1.tags.variableName),
                highlight_1.tags.updateOperator,
                highlight_1.tags.arithmeticOperator,
                highlight_1.tags.logicOperator,
                highlight_1.tags.bitwiseOperator,
                highlight_1.tags.compareOperator,
                highlight_1.tags.function(highlight_1.tags.punctuation),
                highlight_1.tags.punctuation,
                highlight_1.tags.paren,
                highlight_1.tags.derefOperator,
            ],
            color: compass_components_1.codePalette[theme][3],
        },
        {
            tag: [highlight_1.tags.lineComment, highlight_1.tags.blockComment],
            color: compass_components_1.codePalette[theme][2],
            fontStyle: 'italic',
        },
    ], { themeType: theme });
}
const highlightStyles = {
    light: getHighlightStyleForTheme('light'),
    dark: getHighlightStyleForTheme('dark'),
};
function createFoldGutterExtension() {
    return (0, language_1.foldGutter)({
        markerDOM(open) {
            const marker = document.createElement('span');
            marker.className = `foldMarker foldMarker${open ? 'Open' : 'Closed'}`;
            marker.ariaHidden = 'true';
            marker.title = open ? 'Fold code block' : 'Unfold code block';
            marker.innerHTML = open
                ? `<svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 16 16"><path fill="currentColor" d="M8.679 10.796a.554.554 0 0 1-.858 0L4.64 6.976C4.32 6.594 4.582 6 5.069 6h6.362c.487 0 .748.594.43.976l-3.182 3.82z"/></svg>`
                : `<svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 16 16"><path fill="currentColor" d="M10.796 7.321a.554.554 0 0 1 0 .858l-3.82 3.181c-.382.319-.976.058-.976-.429V4.57c0-.487.594-.748.976-.43l3.82 3.182z"/></svg>`;
            return marker;
        },
    });
}
const javascriptExpression = lang_javascript_1.javascriptLanguage.configure({
    top: 'SingleExpression',
});
exports.languages = {
    json: lang_json_1.json,
    javascript() {
        return new language_2.LanguageSupport(javascriptExpression);
    },
};
exports.languageName = state_1.Facet.define({});
function useCodemirrorExtensionCompartment(fn, value, editorViewRef) {
    var _a, _b;
    const extensionCreatorRef = (0, react_1.useRef)(fn);
    extensionCreatorRef.current = fn;
    const compartmentRef = (0, react_1.useRef)();
    (_a = compartmentRef.current) !== null && _a !== void 0 ? _a : (compartmentRef.current = new state_1.Compartment());
    const initialExtensionRef = (0, react_1.useRef)();
    (_b = initialExtensionRef.current) !== null && _b !== void 0 ? _b : (initialExtensionRef.current = compartmentRef.current.of(extensionCreatorRef.current()));
    (0, compass_components_1.useEffectOnChange)(() => {
        var _a, _b;
        (_a = editorViewRef.current) === null || _a === void 0 ? void 0 : _a.dispatch({
            effects: (_b = compartmentRef.current) === null || _b === void 0 ? void 0 : _b.reconfigure(extensionCreatorRef.current()),
        });
    }, value);
    return initialExtensionRef.current;
}
const BaseEditor = ({ initialText: _initialText, text, onChangeText = () => {
}, language = 'json', showLineNumbers = true, showFoldGutter = true, highlightActiveLine: shouldHighlightActiveLine = true, annotations = [], completer, darkMode: _darkMode, className, readOnly = false, onLoad = () => {
}, minLines, maxLines, lineHeight = 16, ...props }) => {
    const darkMode = (0, compass_components_1.useDarkMode)(_darkMode);
    const onChangeTextRef = (0, react_1.useRef)(onChangeText);
    const onLoadRef = (0, react_1.useRef)(onLoad);
    const initialTextProvided = (0, react_1.useRef)(!!_initialText);
    const initialText = (0, react_1.useRef)(_initialText !== null && _initialText !== void 0 ? _initialText : text);
    const initialLanguage = (0, react_1.useRef)(language);
    const containerRef = (0, react_1.useRef)(null);
    const editorViewRef = (0, react_1.useRef)();
    onChangeTextRef.current = onChangeText;
    onLoadRef.current = onLoad;
    const languageExtension = useCodemirrorExtensionCompartment(() => {
        return [exports.languages[language](), exports.languageName.of(language)];
    }, language, editorViewRef);
    const activeLineExtension = useCodemirrorExtensionCompartment(() => {
        return !readOnly && shouldHighlightActiveLine
            ? [(0, view_1.highlightActiveLine)(), (0, view_1.highlightActiveLineGutter)()]
            : [];
    }, [readOnly, shouldHighlightActiveLine], editorViewRef);
    const readOnlyExtension = useCodemirrorExtensionCompartment(() => {
        return state_1.EditorState.readOnly.of(readOnly);
    }, readOnly, editorViewRef);
    const themeConfigExtension = useCodemirrorExtensionCompartment(() => {
        return themeStyles[darkMode ? 'dark' : 'light'];
    }, darkMode, editorViewRef);
    const lineHeightExtension = useCodemirrorExtensionCompartment(() => {
        return view_1.EditorView.theme({
            '.cm-scroller': {
                lineHeight: `${lineHeight}px`,
                ...(maxLines &&
                    maxLines < Infinity && {
                    maxHeight: `${maxLines * lineHeight}px`,
                    overflow: 'auto',
                }),
            },
            '.cm-content, .cm-gutter': {
                ...(minLines && { minHeight: `${minLines * lineHeight}px` }),
            },
        });
    }, [minLines, maxLines, lineHeight], editorViewRef);
    const lineNumbersExtension = useCodemirrorExtensionCompartment(() => {
        return showLineNumbers ? (0, view_1.lineNumbers)() : [];
    }, showLineNumbers, editorViewRef);
    const foldGutterExtension = useCodemirrorExtensionCompartment(() => {
        return showFoldGutter ? createFoldGutterExtension() : [];
    }, showFoldGutter, editorViewRef);
    const annotationsGutterExtension = useCodemirrorExtensionCompartment(() => {
        return annotations.length === 0 ? [] : (0, lint_1.lintGutter)();
    }, annotations.length === 0, editorViewRef);
    const autocomletionExtension = useCodemirrorExtensionCompartment(() => {
        return completer
            ? (0, autocomplete_1.autocompletion)({
                activateOnTyping: true,
                override: [completer],
            })
            : [];
    }, completer, editorViewRef);
    (0, react_1.useLayoutEffect)(() => {
        if (!containerRef.current) {
            throw new Error("Can't mount editor: DOM node is missing");
        }
        const domNode = containerRef.current;
        const editor = (editorViewRef.current = new view_1.EditorView({
            doc: initialText.current,
            extensions: [
                lineNumbersExtension,
                (0, commands_1.history)(),
                foldGutterExtension,
                (0, view_1.drawSelection)(),
                (0, language_1.indentOnInput)(),
                (0, language_1.bracketMatching)(),
                (0, autocomplete_1.closeBrackets)(),
                autocomletionExtension,
                annotationsGutterExtension,
                languageExtension,
                (0, language_2.syntaxHighlighting)(highlightStyles['light']),
                (0, language_2.syntaxHighlighting)(highlightStyles['dark']),
                activeLineExtension,
                lineHeightExtension,
                themeConfigExtension,
                view_1.keymap.of([
                    {
                        key: 'Ctrl-Shift-b',
                        run: prettify,
                    },
                    {
                        key: 'Ctrl-Shift-c',
                        run: copyAll,
                    },
                    ...commands_1.defaultKeymap,
                    ...autocomplete_1.closeBracketsKeymap,
                    ...commands_1.historyKeymap,
                    ...language_1.foldKeymap,
                    ...autocomplete_1.completionKeymap,
                    commands_1.indentWithTab,
                ]),
                readOnlyExtension,
                view_1.EditorView.updateListener.of((update) => {
                    var _a;
                    if (update.docChanged) {
                        const text = (_a = editor.state.sliceDoc()) !== null && _a !== void 0 ? _a : '';
                        onChangeTextRef.current(text, update);
                    }
                }),
            ],
            parent: domNode,
        }));
        domNode._cm = editor;
        onLoadRef.current(editor);
        if (editor && initialLanguage.current === 'json') {
            const docLength = editor.state.doc.length;
            const isSyntaxTreeAvailable = (0, language_1.syntaxTreeAvailable)(editor.state, docLength) ||
                (0, language_1.forceParsing)(editor, docLength, 150);
            if (isSyntaxTreeAvailable) {
                foldAll(editor);
            }
            else {
            }
        }
        return () => {
            delete domNode._cm;
            editor.destroy();
        };
    }, [
        annotationsGutterExtension,
        foldGutterExtension,
        languageExtension,
        lineNumbersExtension,
        readOnlyExtension,
        themeConfigExtension,
        autocomletionExtension,
        lineHeightExtension,
        activeLineExtension,
    ]);
    (0, react_1.useEffect)(() => {
        var _a, _b;
        if (initialTextProvided.current) {
            return;
        }
        if (text !== ((_a = editorViewRef.current) === null || _a === void 0 ? void 0 : _a.state.sliceDoc())) {
            (_b = editorViewRef.current) === null || _b === void 0 ? void 0 : _b.dispatch({
                changes: {
                    from: 0,
                    to: editorViewRef.current.state.doc.length,
                    insert: text,
                },
            });
        }
    }, [text]);
    (0, react_1.useEffect)(() => {
        var _a;
        (_a = editorViewRef.current) === null || _a === void 0 ? void 0 : _a.dispatch({
            effects: lint_1.setDiagnosticsEffect.of(annotations),
        });
    }, [annotations]);
    return (react_1.default.createElement("div", { ref: containerRef, className: (0, compass_components_1.cx)(editorStyle, className), "data-codemirror": "true", ...props }));
};
exports.JSONEditor = BaseEditor;
function isTopNode(node) {
    return !node
        ? true
        : node.name === 'Array' || node.name === 'Property'
            ? false
            : node.name === 'JsonText'
                ? true
                : isTopNode(node.parent);
}
const foldAll = (editor) => {
    const foldableProperties = [];
    (0, language_1.syntaxTree)(editor.state).iterate({
        enter(nodeRef) {
            if (['{', '['].includes(String(nodeRef.name)) &&
                (0, language_1.foldable)(editor.state, nodeRef.from, nodeRef.to) &&
                !isTopNode(nodeRef.node)) {
                if (!nodeRef.node.parent) {
                    throw new Error('Trying to fold node without parent');
                }
                const range = (0, language_1.foldInside)(nodeRef.node.parent);
                if (range) {
                    foldableProperties.push(range);
                }
                return false;
            }
        },
    });
    if (foldableProperties.length > 0) {
        editor.dispatch({
            effects: foldableProperties.map((range) => {
                return language_1.foldEffect.of(range);
            }),
        });
    }
    return !!foldableProperties.length;
};
const copyAll = (editorView) => {
    void navigator.clipboard.writeText(editorView.state.sliceDoc());
    return true;
};
const prettify = (editorView) => {
    if (editorView.state.facet(state_1.EditorState.readOnly)) {
        return false;
    }
    const language = editorView.state.facet(exports.languageName)[0];
    const doc = editorView.state.sliceDoc();
    try {
        const formatted = (0, prettify_1.prettify)(doc, language === 'json' ? 'json' : 'javascript-expression');
        if (formatted !== doc) {
            editorView.dispatch({
                changes: {
                    from: 0,
                    to: editorView.state.doc.length,
                    insert: formatted,
                },
            });
            return true;
        }
    }
    catch (_a) {
    }
    return false;
};
BaseEditor.foldAll = foldAll;
BaseEditor.unfoldAll = language_1.unfoldAll;
BaseEditor.copyAll = copyAll;
BaseEditor.prettify = prettify;
//# sourceMappingURL=json-editor.js.map